@@ -42,11 +42,8 @@
 #|b|y| |D|a|n|i|e|l| |B|u|s|c|o|m|b|e|
 #+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
 #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-#|d|b|u|s|c|o|m|b|e|@|u|s|g|s|.|g|o|v|
+#|d|a|n|i|e|l|.|b|u|s|c|o|m|b|e|@|n|a|u|.|e|d|u|
 #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
-#|U|.|S|.| |G|e|o|l|o|g|i|c|a|l| |S|u|r|v|e|y|
-#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
 
 #"""
 
@@ -55,6 +52,7 @@
 # =========================================================
 
 #operational
+from __future__ import print_function
 from __future__ import division
 from scipy.io import savemat, loadmat
 import os, time #, sys, getopt
@@ -73,6 +71,7 @@ import PyHum.ppdrc as ppdrc
 
 from scipy.special import jv
 from scipy.ndimage.filters import median_filter
+from skimage.restoration import denoise_tv_chambolle
 
 #plotting
 import matplotlib.pyplot as plt
@@ -91,7 +90,7 @@ warnings.filterwarnings("ignore")
 # ========================================================
 
 #################################################
-def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0, ph = 7, temp = 10, salinity = 0, dconcfile = None):
+def correct(humfile, sonpath, maxW, doplot, dofilt, correct_withwater, ph, temp, salinity, dconcfile):
 
     '''
     Remove water column and carry out some rudimentary radiometric corrections, 
@@ -186,59 +185,59 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
 
     # prompt user to supply file if no input file given
     if not humfile:
-      print 'An input file is required!!!!!!'
+      print('An input file is required!!!!!!')
       Tk().withdraw() # we don't want a full GUI, so keep the root window from appearing
       inputfile = askopenfilename(filetypes=[("DAT files","*.DAT")]) 
 
     # prompt user to supply directory if no input sonpath is given
     if not sonpath:
-      print 'A *.SON directory is required!!!!!!'
+      print('A *.SON directory is required!!!!!!')
       Tk().withdraw() # we don't want a full GUI, so keep the root window from appearing
       sonpath = askdirectory() 
 
     # print given arguments to screen and convert data type where necessary
     if humfile:
-      print 'Input file is %s' % (humfile)
+      print('Input file is %s' % (humfile))
 
     if sonpath:
-      print 'Sonar file path is %s' % (sonpath)
+      print('Sonar file path is %s' % (sonpath))
 
     if maxW:
       maxW = np.asarray(maxW,float)
-      print 'Max. transducer power is %s W' % (str(maxW))
+      print('Max. transducer power is %s W' % (str(maxW)))
 
     if doplot:
       doplot = int(doplot)
       if doplot==0:
-         print "Plots will not be made"
+         print("Plots will not be made")
 
     if dofilt:
       dofilt = int(dofilt)
       if dofilt==0:
-         print "Phase preserving filter will not be applied"
+         print("Phase preserving filter will not be applied")
       else:
-         print "Phase preserving filter will be applied"
+         print("Phase preserving filter will be applied")
 
     if correct_withwater:
       correct_withwater = int(correct_withwater)
       if correct_withwater==1:
-         print "Correction will be applied without removing water column"
+         print("Correction will be applied without removing water column")
 
     if salinity:
        salinity = np.asarray(salinity,float)
-       print 'Salinity is %s ppt' % (str(salinity))
+       print('Salinity is %s ppt' % (str(salinity)))
 
     if ph:
        ph = np.asarray(ph,float)
-       print 'pH is %s' % (str(ph))
+       print('pH is %s' % (str(ph)))
 
     if temp:
        temp = np.asarray(temp,float)
-       print 'Temperature is %s' % (str(temp))
+       print('Temperature is %s' % (str(temp)))
 
     if dconcfile is not None:
        try:
-          print 'Suspended sediment size/conc. file is %s' % (dconcfile)
+          print('Suspended sediment size/conc. file is %s' % (dconcfile))
           dconc = np.genfromtxt(dconcfile).T
           conc = dconc[1]
           dens = dconc[2]
@@ -377,8 +376,13 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
     Zt2 = Zt-avg + np.nanmean(avg)
     Zt2 = Zt2 + np.abs(np.nanmin(Zt2))
 
+    try:
+       Zt2 = median_filter(Zt2, (3,3))
+    except:
+       pass
+
     ##Zt2 = np.empty(np.shape(Zt)) 
-    ##for kk in xrange(np.shape(Zt)[1]):
+    ##for kk in range(np.shape(Zt)[1]):
     ##   Zt2[:,kk] = (Zt[:,kk] - avg) + np.nanmean(avg)
     ##Zt2[Zt<=0] = np.nan
     ##Zt2[Zt2<=0] = np.nan    
@@ -419,7 +423,7 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
     Zt2 = Zt2 + np.abs(np.nanmin(Zt2))
 
     ##Zt2 = np.empty(np.shape(Zt))
-    ##for kk in xrange(np.shape(Zt)[1]):
+    ##for kk in range(np.shape(Zt)[1]):
     ##   Zt2[:,kk] = (Zt[:,kk] - avg) + np.nanmean(avg)
     ##Zt2[Zt<=0] = np.nan
     ##Zt2[Zt2<=0] = np.nan    
@@ -441,7 +445,7 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
           star_fpw = io.get_mmap_data(sonpath, base, '_data_star_lw.dat', 'float32', shape_star) 
           
           if len(np.shape(star_fpw))>2:
-             for p in xrange(len(star_fpw)):
+             for p in range(len(star_fpw)):
                 plot_merged_scans(port_fpw[p], star_fpw[p], dist_m, shape_port, ft, sonpath, p)
           else:
              plot_merged_scans(port_fpw, star_fpw, dist_m, shape_port, ft, sonpath, 0)
@@ -449,7 +453,7 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
        else:
 
           if len(np.shape(star_fp))>2:
-             for p in xrange(len(star_fp)):
+             for p in range(len(star_fp)):
                 plot_merged_scans(port_fp[p], star_fp[p], dist_m, shape_port, ft, sonpath, p)
           else:
              plot_merged_scans(port_fp, star_fp, dist_m, shape_port, ft, sonpath, 0)
@@ -530,7 +534,7 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
        
        if doplot==1:
           if len(np.shape(low_fp))>2:
-             for p in xrange(len(low_fp)):
+             for p in range(len(low_fp)):
                 plot_dwnlow_scans(low_fp[p], dist_m, shape_low, ft, sonpath, p)
           else:
              plot_dwnlow_scans(low_fp, dist_m, shape_low, ft, sonpath, 0)
@@ -558,7 +562,7 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
        
        if doplot==1:
           if len(np.shape(hi_fp))>2:
-             for p in xrange(len(hi_fp)):
+             for p in range(len(hi_fp)):
                 plot_dwnhi_scans(hi_fp[p], dist_m, shape_hi, ft, sonpath, p)
           else:
              plot_dwnhi_scans(hi_fp, dist_m, shape_hi, ft, sonpath, 0)
@@ -567,10 +571,10 @@ def correct(humfile, sonpath, maxW=1000, doplot=1, dofilt=0, correct_withwater=0
        elapsed = (time.time() - start)
     else: # windows
        elapsed = (time.clock() - start)
-    print "Processing took ", elapsed , "seconds to analyse"
-
-    print "Done!"
+    print("Processing took "+ str(elapsed) + "seconds to analyse")
 
+    print("Done!")
+    print("===================================================")
 
 # =========================================================
 def water_atten(H,f,c,pH,T,S):
@@ -637,13 +641,13 @@ def remove_water(fp,bed,shape, dep_m, pix_m, calcR,  maxW):
        A = []
 
     if  len(np.shape(fp))>2:
-       for p in xrange(len(fp)):
+       for p in range(len(fp)):
           data_dB = fp[p]*(10*np.log10(maxW)/255)
 
           Zbed = np.squeeze(bed[shape[-1]*p:shape[-1]*(p+1)])
 
           # shift proportionally depending on where the bed is
-          for k in xrange(np.shape(data_dB)[1]):
+          for k in range(np.shape(data_dB)[1]):
              try:
                 data_dB[:,k] = np.r_[data_dB[Zbed[k]:,k], np.zeros( (np.shape(data_dB)[0] -  np.shape(data_dB[Zbed[k]:,k])[0] ,) )]
              except:
@@ -665,7 +669,7 @@ def remove_water(fp,bed,shape, dep_m, pix_m, calcR,  maxW):
                 r[:,k] = np.sqrt(yvec**2 - d[k]**2)
 
              # shift proportionally depending on where the bed is
-             for k in xrange(np.shape(r)[1]):
+             for k in range(np.shape(r)[1]):
                 try:
                    r[:,k] = np.r_[r[Zbed[k]:,k], np.zeros( (np.shape(r)[0] -  np.shape(r[Zbed[k]:,k])[0] ,) )]
                    a[:,k] = np.r_[a[Zbed[k]:,k], np.zeros( (np.shape(a)[0] -  np.shape(a[Zbed[k]:,k])[0] ,) )]
@@ -683,7 +687,7 @@ def remove_water(fp,bed,shape, dep_m, pix_m, calcR,  maxW):
        Zbed = np.squeeze(bed)
 
        # shift proportionally depending on where the bed is
-       for k in xrange(np.shape(data_dB)[1]):
+       for k in range(np.shape(data_dB)[1]):
           try:
             data_dB[:,k] = np.r_[data_dB[Zbed[k]:,k], np.zeros( (np.shape(data_dB)[0] -  np.shape(data_dB[Zbed[k]:,k])[0] ,) )]
           except:
@@ -705,7 +709,7 @@ def remove_water(fp,bed,shape, dep_m, pix_m, calcR,  maxW):
              r[:,k] = np.sqrt(yvec**2 - d[k]**2)
 
           # shift proportionally depending on where the bed is
-          for k in xrange(np.shape(r)[1]):
+          for k in range(np.shape(r)[1]):
              try:
                 r[:,k] = np.r_[r[Zbed[k]:,k], np.zeros( (np.shape(r)[0] -  np.shape(r[Zbed[k]:,k])[0] ,) )]
                 a[:,k] = np.r_[a[Zbed[k]:,k], np.zeros( (np.shape(a)[0] -  np.shape(a[Zbed[k]:,k])[0] ,) )]
@@ -726,7 +730,7 @@ def correct_scans(fp, a_fp, TL, dofilt):
     if np.ndim(fp)==2:
        return c_scans(fp, a_fp, TL, dofilt)
     else:
-       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans)(fp[p], a_fp[p], TL[p], dofilt) for p in xrange(len(fp)))
+       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans)(fp[p], a_fp[p], TL[p], dofilt) for p in range(len(fp)))
 
 # =========================================================
 def c_scans(fp, a_fp, TL, dofilt):
@@ -746,7 +750,7 @@ def correct_scans_lambertian(fp, a_fp, TL, R, c, f, theta, alpha):
     if np.ndim(fp)==2:
        return c_scans_lambertian(fp, a_fp, TL, R, c, f, theta, alpha)
     else:
-       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans_lambertian)(fp[p], a_fp[p], TL[p], R[p], c, f, theta, alpha) for p in xrange(len(fp)))
+       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans_lambertian)(fp[p], a_fp[p], TL[p], R[p], c, f, theta, alpha) for p in range(len(fp)))
        
 # =========================================================
 def c_scans_lambertian(fp, a_fp, TL, R, c, f, theta, alpha):
@@ -786,7 +790,11 @@ def c_scans_lambertian(fp, a_fp, TL, R, c, f, theta, alpha):
    mg = 10**np.log10(mg + TL)
    mg[fp==0] = np.nan
    mg[mg<0] = np.nan
-   
+
+   mask = np.isnan(mg)
+   mg[np.isnan(mg)] = 0
+   mg = denoise_tv_chambolle(mg.copy(), weight=.2, multichannel=False).astype('float32')
+   mg[mask==True] = np.nan
    return mg   
 
 # =========================================================
@@ -794,7 +802,7 @@ def correct_scans2(fp, TL):
     if np.ndim(fp)==2:
        return c_scans2(fp, TL)
     else:
-       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans2)(fp[p], TL[p]) for p in xrange(len(fp)))
+       return Parallel(n_jobs = cpu_count(), verbose=0)(delayed(c_scans2)(fp[p], TL[p]) for p in range(len(fp)))
 
 # =========================================================
 def c_scans2(fp, TL):
@@ -878,8 +886,6 @@ def plot_dwnhi_scans(dat_dwnhi, dist_m, shape_hi, ft, sonpath, p):
        del fig
 
 
-
-
 # =========================================================
 # =========================================================
 if __name__ == '__main__':
