@@ -42,11 +42,8 @@
 #|b|y| |D|a|n|i|e|l| |B|u|s|c|o|m|b|e|
 #+-+-+ +-+-+-+-+-+-+ +-+-+-+-+-+-+-+-+
 #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-#|d|b|u|s|c|o|m|b|e|@|u|s|g|s|.|g|o|v|
+#|d|a|n|i|e|l|.|b|u|s|c|o|m|b|e|@|n|a|u|.|e|d|u|
 #+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
-#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
-#|U|.|S|.| |G|e|o|l|o|g|i|c|a|l| |S|u|r|v|e|y|
-#+-+-+-+-+ +-+-+-+-+-+-+-+-+-+-+ +-+-+-+-+-+-+
 
 #"""
 
@@ -55,6 +52,7 @@
 # =========================================================
 
 # operational
+from __future__ import print_function
 import os, time #sys, getopt, 
 from scipy.io import loadmat #, savemat
 from joblib import Parallel, delayed, cpu_count
@@ -93,7 +91,7 @@ import warnings
 warnings.filterwarnings("ignore")
 
 #################################################
-def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasses=4, maxscale=20, notes=4):
+def texture(humfile, sonpath, win, shift, doplot, density, numclasses, maxscale, notes):
           
       '''
       Create a texture lengthscale map using the algorithm detailed by Buscombe et al. (2015)
@@ -143,54 +141,54 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
 
       # prompt user to supply file if no input file given
       if not humfile:
-         print 'An input file is required!!!!!!'
+         print('An input file is required!!!!!!')
          Tk().withdraw() # we don't want a full GUI, so keep the root window from appearing
          humfile = askopenfilename(filetypes=[("DAT files","*.DAT")]) 
 
       # prompt user to supply directory if no input sonpath is given
       if not sonpath:
-         print 'A *.SON directory is required!!!!!!'
+         print('A *.SON directory is required!!!!!!')
          Tk().withdraw() # we don't want a full GUI, so keep the root window from appearing
          sonpath = askdirectory() 
 
       # print given arguments to screen and convert data type where necessary
       if humfile:
-         print 'Input file is %s' % (humfile)
+         print('Input file is %s' % (humfile))
          
       if sonpath:
-         print 'Sonar file path is %s' % (sonpath)
+         print('Sonar file path is %s' % (sonpath))
          
       if win:
          win = np.asarray(win,int)
-         print 'Window is %s square pixels' % (str(win))
+         print('Window is %s square pixels' % (str(win)))
          
       if shift:
          shift = np.asarray(shift,int)
-         print 'Min shift is %s pixels' % (str(shift))
+         print('Min shift is %s pixels' % (str(shift)))
          
       if density:
          density = np.asarray(density,int)
-         print 'Image will be sampled every %s pixels' % (str(density))
+         print('Image will be sampled every %s pixels' % (str(density)))
          
       if numclasses:
          numclasses = np.asarray(numclasses,int)
-         print 'Number of sediment classes: %s' % (str(numclasses))
+         print('Number of sediment classes: %s' % (str(numclasses)))
          
       if maxscale:
          maxscale = np.asarray(maxscale,int)
-         print 'Max scale as inverse fraction of data length: %s' % (str(maxscale))
+         print('Max scale as inverse fraction of data length: %s' % (str(maxscale)))
          
       if notes:
          notes = np.asarray(notes,int)
-         print 'Notes per octave: %s' % (str(notes))
+         print('Notes per octave: %s' % (str(notes)))
          
       if doplot:
          doplot = int(doplot)
          if doplot==0:
-            print "Plots will not be made"    
+            print("Plots will not be made")
       
       
-      print '[Default] Number of processors is %s' % (str(cpu_count()))
+      print('[Default] Number of processors is %s' % (str(cpu_count())))
                         
       ########################################################
       ########################################################
@@ -219,7 +217,7 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
       dist_m = np.squeeze(meta['dist_m'])
 
       ### port
-      print "processing port side ..."
+      print("processing port side ...")
       # load memory mapped scan ... port
       shape_port = np.squeeze(meta['shape_port'])
       if shape_port!='':
@@ -232,7 +230,7 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
          port_fp2 = io.get_mmap_data(sonpath, base, '_data_port_l.dat', 'float32', tuple(shape_port))
 
       ### star
-      print "processing starboard side ..."
+      print("processing starboard side ...")
       # load memory mapped scan ... port
       shape_star = np.squeeze(loadmat(sonpath+base+'meta.mat')['shape_star'])
       if shape_star!='':
@@ -259,7 +257,7 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
 
       if len(shape_star)>2:
          #SRT = []
-         for p in xrange(len(port_fp)):
+         for p in range(len(port_fp)):
             
             Z,ind = humutils.sliding_window_sliced(np.vstack((np.flipud(port_fp[p]), star_fp[p])), density, (win,win),(shift,shift))
             
@@ -359,13 +357,13 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
       if doplot==1:
 
          if len(shape_star)>2:
-            for p in xrange(len(star_fp)):
+            for p in range(len(star_fp)):
                plot_class(dist_m, shape_port, port_fp[p], star_fp[p], class_fp[p], ft, humfile, sonpath, base, p)
          else:
             plot_class(dist_m, shape_port, port_fp, star_fp, class_fp, ft, humfile, sonpath, base, 0)
 
          if len(shape_star)>2:
-            for p in xrange(len(star_fp)):
+            for p in range(len(star_fp)):
                plot_contours(dist_m, shape_port, class_fp[p], ft, humfile, sonpath, base, numclasses, p)
          else:
             plot_contours(dist_m, shape_port, class_fp, ft, humfile, sonpath, base, numclasses, 0)
@@ -378,7 +376,7 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
          with open(os.path.normpath(os.path.join(sonpath,base+'_data_kclass.dat')), 'w+') as ff:
             fp = np.memmap(ff, dtype='float32', mode='w+', shape=tuple(shape))
 
-         for p in xrange(len(port_fp)):
+         for p in range(len(port_fp)):
             wc = get_kclass(class_fp[p].copy(), numclasses)
             fp[p] = wc.astype('float32')
             del wc
@@ -402,7 +400,7 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
       if doplot==1:
 
          if len(shape_star)>2:
-            for p in xrange(len(star_fp)):
+            for p in range(len(star_fp)):
                plot_kmeans(dist_m, shape_port, port_fp[p], star_fp[p], kclass_fp[p], ft, humfile, sonpath, base, p)
          else:
             plot_kmeans(dist_m, shape_port, port_fp, star_fp, kclass_fp, ft, humfile, sonpath, base, 0)         
@@ -411,21 +409,21 @@ def texture(humfile, sonpath, win=100, shift=10, doplot=1, density=50, numclasse
          elapsed = (time.time() - start)
       else: # windows
          elapsed = (time.clock() - start)
-      print "Processing took ", elapsed , "seconds to analyse"
+      print("Processing took "+ str(elapsed)+ "seconds to analyse")
 
-      print "Done!"
-    
+      print("Done!")
+      print("===================================================")    
     
 # =========================================================
 def get_srt(Z,ind,maxscale, notes, win): #, density):    
     try:
        #print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
-       print "%s windows to process" % (str(len(Z)))                              
+       print("%s windows to process" % (str(len(Z))))                         
        # do the wavelet clacs and get the stats
-       d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win) for k in xrange(len(Z))) #density
+       d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win) for k in range(len(Z))) #density
     except:
-       print "memory error: trying serial"
-       d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win) for k in xrange(len(Z))) #density
+       print("memory error: trying serial")
+       d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win) for k in range(len(Z))) #density
 
     srt = np.reshape(d , ( ind[0], ind[1] ) )
     del d
@@ -461,7 +459,7 @@ def plot_class(dist_m, shape_port, dat_port, dat_star, dat_class, ft, humfile, s
       Zdist = dist_m
       extent = shape_port[0]
 
-   print "Plotting ... "
+   print("Plotting ... ")
    # create fig 1
    fig = plt.figure()
    fig.subplots_adjust(wspace = 0, hspace=0.075)
@@ -611,10 +609,10 @@ if __name__ == '__main__':
 #            try:
 #               print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
 #            # do the wavelet clacs and get the stats
-#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in range(len(Z)))
 #            except:
 #               print "memory error: trying serial"
-#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in range(len(Z)))
 
 #            srt = np.reshape(d , ( ind[0], ind[1] ) )
 #            del d
@@ -622,10 +620,10 @@ if __name__ == '__main__':
 #            try:
 #               print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
 #            # do the wavelet clacs and get the stats
-#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in range(len(Z)))
 #            except:
 #               print "memory error: trying serial"
-#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in range(len(Z)))
 
 #            srt2 = np.reshape(d , ( ind[0], ind[1] ) )
 #            del d
@@ -640,10 +638,10 @@ if __name__ == '__main__':
 #            try:
 #               print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
 #            # do the wavelet clacs and get the stats
-#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in range(len(Z)))
 #            except:
 #               print "memory error: trying serial"
-#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k], maxscale, notes, win, density) for k in range(len(Z)))
 
 #            srt = np.reshape(d , ( ind[0], ind[1] ) )
 #            del d
@@ -651,10 +649,10 @@ if __name__ == '__main__':
 #            try:
 #               print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
 #            # do the wavelet clacs and get the stats
-#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in range(len(Z)))
 #            except:
 #               print "memory error: trying serial"
-#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in xrange(len(Z)))
+#               d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win, density) for k in range(len(Z)))
 
 #            srt2 = np.reshape(d , ( ind[0], ind[1] ) )
 #            del d
@@ -676,10 +674,10 @@ if __name__ == '__main__':
             #   #print "%s windows to process with a density of %s" % (str(len(Z)), str(density)) #% (str(len(Z)), str(density))
             #   print "%s windows to process" % (str(len(Z)))               
             ## do the wavelet clacs and get the stats
-            #   d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win) for k in xrange(len(Z))) #density
+            #   d = Parallel(n_jobs = -1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win) for k in range(len(Z))) #density
             #except:
             #   print "memory error: trying serial"
-            #   d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win) for k in xrange(len(Z))) #density
+            #   d = Parallel(n_jobs = 1, verbose=0)(delayed(parallel_me)(Z[k].T, maxscale, notes, win) for k in range(len(Z))) #density
 
             #srt2 = np.reshape(d , ( ind[0], ind[1] ) )
             #del d
